"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// @ts-ignore
const Server_1 = __importDefault(require("@hyext/matrix/src/Server"));
// @ts-ignore
const bundle_1 = __importDefault(require("@hyext/matrix/src/shared/output/bundle"));
// @ts-ignore
const metro_config_1 = require("metro-config");
const path_1 = __importDefault(require("path"));
const getDefaultConfig_1 = __importDefault(require("../matrix-config/getDefaultConfig"));
const saveAssets_1 = __importDefault(require("./saveAssets"));
const logger_1 = __importDefault(require("../lib/logger"));
const md5_1 = __importDefault(require("../lib/md5"));
let totalCompileTime = 0;
function compile({ projectRoot, filterModules, imageBaseURL, SDKAlias, metroConfig = {}, bundleConfigs, extUuid, cacheVersion, }) {
    return __awaiter(this, void 0, void 0, function* () {
        const allStart = Date.now();
        const defaultConfig = getDefaultConfig_1.default({
            projectRoot,
            imageBaseURL,
            filterModules: filterModules || new Set(),
            SDKAlias,
            mode: 'production',
            extUuid,
            cacheVersion,
        });
        const finalConfig = metro_config_1.mergeConfig(defaultConfig, metroConfig);
        const server = new Server_1.default(Object.assign(Object.assign({}, finalConfig), { extUuid }));
        const bundlePlatform = ({ entryAbsPath, outputDir, assetsDir, getBundleName, }, platform) => __awaiter(this, void 0, void 0, function* () {
            const bundleStart = Date.now();
            const bundleName = getBundleName ? getBundleName(platform, entryAbsPath) : `${path_1.default.basename(entryAbsPath, '.js')}.${platform}.jsbundle`;
            const bundleAbsPath = path_1.default.resolve(outputDir, bundleName);
            const requestOpts = {
                entryFile: entryAbsPath,
                sourceMapUrl: `${path_1.default.basename(bundleAbsPath)}.map`,
                dev: false,
                minify: true,
                platform,
            };
            const buildStart = Date.now();
            const bundle = yield bundle_1.default.build(server, requestOpts);
            console.log(`build ${path_1.default.basename(entryAbsPath)}, ${platform}: ${Date.now() - buildStart}ms`);
            const outputOpts = {
                bundleOutput: bundleAbsPath,
                bundleEncoding: 'utf8',
                dev: false,
                indexedRamBundle: false,
                platform,
                assetsDest: assetsDir || outputDir,
                sourcemapOutput: `${bundleAbsPath}.map`,
            };
            const saveStart = Date.now();
            yield bundle_1.default.save(bundle, outputOpts, logger_1.default.info);
            if (platform === 'ios') {
                const outputAssets = yield server.getAssets(Object.assign(Object.assign(Object.assign({}, Server_1.default.DEFAULT_BUNDLE_OPTIONS), requestOpts), { bundleType: 'todo' }));
                // 只需要保存一份资源即可
                yield saveAssets_1.default(outputAssets, platform, assetsDir || outputDir);
            }
            console.log(`save ${path_1.default.basename(entryAbsPath)}, ${platform}: ${Date.now() - saveStart}ms`);
            const bundleMd5 = yield md5_1.default(bundleAbsPath);
            console.log(`build+save for ${path_1.default.basename(entryAbsPath)} ${platform}: ${Date.now() - bundleStart}ms`);
            return {
                entryAbsPath,
                bundleAbsPath,
                bundleMd5,
                platform,
            };
        });
        const promises = [];
        bundleConfigs.forEach(bundleConfig => {
            (bundleConfig.platforms || ['ios', 'android']).forEach(platform => {
                promises.push(bundlePlatform(bundleConfig, platform));
            });
        });
        try {
            const compileOutputs = yield Promise.all(promises);
            totalCompileTime += (Date.now() - allStart);
            return compileOutputs;
        }
        finally {
            console.log(`totalCompileTime: ${totalCompileTime}`);
            server.end();
        }
    });
}
exports.default = compile;
